'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _webpackManifestPlugin = require('webpack-manifest-plugin');

var _webpackManifestPlugin2 = _interopRequireDefault(_webpackManifestPlugin);

var _stylelintWebpackPlugin = require('stylelint-webpack-plugin');

var _stylelintWebpackPlugin2 = _interopRequireDefault(_stylelintWebpackPlugin);

var _path = require('path');

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _Modernizr = require('./Plugins/Modernizr');

var _Modernizr2 = _interopRequireDefault(_Modernizr);

var _UglifyJs = require('./Plugins/UglifyJs');

var _UglifyJs2 = _interopRequireDefault(_UglifyJs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Hide DeprecationWarning: loaderUtils.parseQuery()
// received a non-string value which can be problematic
/*
 * This file is part of the Distribution library.
 *
 * Copyright (c) 2015-2016 LIN3S <info@lin3s.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @author Beñat Espiña <bespina@lin3s.com>
 */

process.noDeprecation = true;

var rootPath = './../../../..',
    esLintDefaultOptions = {
  configFile: (0, _path.join)(__dirname, rootPath + '/.eslintrc.js')
},
    styleLintDefaultOptions = {
  configFile: (0, _path.join)(__dirname, rootPath + '/.stylelintrc.js'),
  sintax: 'scss'
};

var isProdEnvironment = function isProdEnvironment(options) {
  return typeof options !== 'undefined' && options.env === 'prod';
};

var optionsOf = function optionsOf(plugin, options) {
  var defaultOptions = void 0;
  if (typeof options.plugins !== 'undefined' && typeof options.plugins[plugin] !== 'undefined') {
    defaultOptions = options.plugins[plugin];
  }

  return defaultOptions;
};

var cssFilename = function cssFilename(options) {
  var name = isProdEnvironment(options) === true && typeof options.output.cssFilenameProduction !== 'undefined' ? options.output.cssFilenameProduction : options.output.cssFilename;

  return options.output.cssPath + '/' + name;
};

var jsFilename = function jsFilename(options) {
  return isProdEnvironment(options) === true && typeof options.output.jsFilenameProduction !== 'undefined' ? options.output.jsFilenameProduction : options.output.jsFilename;
};

var getPlugins = function getPlugins(options) {
  var plugins = [new _stylelintWebpackPlugin2.default(styleLintDefaultOptions), new _extractTextWebpackPlugin2.default(cssFilename(options)), new _webpack2.default.LoaderOptionsPlugin({
    options: {
      sassLoader: {
        includePaths: [(0, _path.join)(__dirname, options.input.scss)]
      },
      eslint: typeof options.eslint === 'undefined' ? esLintDefaultOptions : optionsOf('eslint', options)
    }
  })];

  if (typeof options.manifest !== 'undefined') {
    plugins.push(new _webpackManifestPlugin2.default({ fileName: options.manifest }));
  }

  if (isProdEnvironment(options)) {
    plugins.push((0, _UglifyJs2.default)(optionsOf('uglifyjs', options)));
    plugins.push((0, _Modernizr2.default)(optionsOf('modernizr', options)));
  }

  return plugins;
};

var getRules = function getRules(include, options) {
  var rules = [{
    test: /\.jsx?$/,
    include: include,
    exclude: /(node_modules|bower_components)/,
    use: [{
      loader: 'babel-loader',
      options: {
        presets: ["react", "es2015", "stage-2"],
        compact: false
      }
    }, {
      loader: 'eslint-loader',
      options: {
        enforce: 'pre'
      }
    }]
  }, {
    test: /\.json$/,
    use: 'json-loader'
  }, {
    test: /\.(s?css)$/,
    use: _extractTextWebpackPlugin2.default.extract({
      publicPath: typeof options.output.cssPublicPath === 'undefined' ? '/' : options.output.cssPublicPath,
      fallback: 'style-loader',
      use: [{
        loader: 'css-loader'
      }, {
        loader: 'postcss-loader',
        options: {
          plugins: function plugins() {
            return [(0, _autoprefixer2.default)(options.postcss.autoprefixer)];
          }
        }
      }, {
        loader: 'sass-loader'
      }]
    })
  }];

  if (typeof options.module !== 'undefined' && typeof options.module.rules !== 'undefined') {
    if (options.module.rules instanceof Array) {
      options.module.rules.forEach(function (rule) {
        rules.push(rule);
      });
    } else {
      rules.push(options.module.rules);
    }
  }

  return rules;
};

exports.default = function (customOptions) {
  var include = (0, _path.join)(__dirname, rootPath + '/' + customOptions.input.base);

  return function (webpackOptions) {
    var options = typeof webpackOptions === 'undefined' ? customOptions : Object.assign(webpackOptions, customOptions);

    return {
      entry: options.entry,
      output: {
        path: (0, _path.join)(__dirname, rootPath + '/' + options.output.jsPath),
        publicPath: typeof options.output.jsPublicPath === 'undefined' ? options.output.jsPath : options.output.jsPublicPath,
        filename: jsFilename(options)
      },
      module: {
        rules: getRules(include, options)
      },
      resolve: {
        modules: [include, 'node_modules'],
        extensions: ['.js', '.json', '.jsx', '.css', '.scss', '.svg'],
        alias: options.alias
      },
      plugins: getPlugins(options),
      devtool: isProdEnvironment(options) ? false : 'source-map'
    };
  };
};